"use strict";Object.defineProperty(exports, "__esModule", {value: true}); function _optionalChain(ops) { let lastAccessLHS = undefined; let value = ops[0]; let i = 1; while (i < ops.length) { const op = ops[i]; const fn = ops[i + 1]; i += 2; if ((op === 'optionalAccess' || op === 'optionalCall') && value == null) { return undefined; } if (op === 'access' || op === 'optionalAccess') { lastAccessLHS = value; value = fn(value); } else if (op === 'call' || op === 'optionalCall') { value = fn((...args) => value.call(lastAccessLHS, ...args)); lastAccessLHS = undefined; } } return value; }// src/list-markdown.ts
var listMarkdownPlugin = (md, _options) => {
  md.core.ruler.after("inline", "bangle-list-kind-attrs", (state) => {
    const tokens = state.tokens;
    const listInfoStack = [];
    for (let i = 0; i < tokens.length; i++) {
      const token = tokens[i];
      if (!token) continue;
      const isBulletOpen = isBulletListOpen(token);
      const isOrderedOpen = isOrderedListOpen(token);
      if (isBulletOpen || isOrderedOpen) {
        const kind = isBulletOpen ? "bullet" : "ordered";
        const tight = token.attrGet("tight") === "true";
        listInfoStack.push({ kind, tight });
        token.attrSet("data-bangle-list-kind", kind);
      } else if (token.type === "bullet_list_close" || token.type === "ordered_list_close") {
        if (listInfoStack.length > 0) {
          listInfoStack.pop();
        } else {
          console.warn("Unbalanced list tokens:", token.type);
        }
      } else if (token.type === "list_item_open") {
        const currentListInfo = listInfoStack[listInfoStack.length - 1];
        if (currentListInfo) {
          if (!token.attrGet("data-bangle-list-kind")) {
            token.attrSet("data-bangle-list-kind", currentListInfo.kind);
          }
          token.attrSet("data-bangle-list-tight", String(currentListInfo.tight));
        }
      }
    }
    return false;
  });
  md.core.ruler.after("bangle-list-kind-attrs", "bangle-task-lists", (state) => {
    const tokens = state.tokens;
    for (let i = 2; i < tokens.length; i++) {
      if (isTodoItem(tokens, i)) {
        convertToTaskItem(tokens, i);
      }
    }
    return false;
  });
  const originalRenderToken = md.renderer.renderToken.bind(md.renderer);
  md.renderer.renderToken = (tokens, idx, options) => {
    const token = tokens[idx];
    if (_optionalChain([token, 'optionalAccess', _ => _.type]) === "list_item_open") {
      const kindAttr = token.attrGet("data-bangle-list-kind");
      if (kindAttr === "task") {
        const checkedAttr = token.attrGet("data-bangle-task-checked") || "false";
        token.attrSet("data-bangle-task-checked", checkedAttr);
      }
    }
    return originalRenderToken(tokens, idx, options);
  };
  md.core.ruler.after(
    "bangle-task-lists",
    "bangle-ordered-list-order",
    (state) => {
      const tokens = state.tokens;
      const orderStack = [];
      for (const token of tokens) {
        switch (token.type) {
          case "ordered_list_open": {
            let start = 1;
            const startAttr = token.attrGet("start");
            if (startAttr != null && !Number.isNaN(Number(startAttr))) {
              start = Number.parseInt(startAttr, 10);
            } else if (token.markup) {
              const match = token.markup.match(/^(\d+)/);
              start = match ? Number.parseInt(match[1], 10) : 1;
            }
            orderStack.push(start);
            if (!token.attrGet("data-bangle-list-kind")) {
              token.attrSet("data-bangle-list-kind", "ordered");
            }
            break;
          }
          case "ordered_list_close": {
            orderStack.pop();
            break;
          }
          case "list_item_open": {
            if (token.attrGet("data-bangle-list-kind") === "ordered") {
              const currentOrder = orderStack[orderStack.length - 1];
              if (orderStack.length > 0 && currentOrder !== void 0) {
                token.attrSet("data-bangle-list-order", String(currentOrder));
                orderStack[orderStack.length - 1] = currentOrder + 1;
              }
            }
            break;
          }
          default:
            break;
        }
      }
      return false;
    }
  );
};
function isOrderedListOpen(token) {
  return _optionalChain([token, 'optionalAccess', _2 => _2.type]) === "ordered_list_open";
}
function isBulletListOpen(token) {
  return _optionalChain([token, 'optionalAccess', _3 => _3.type]) === "bullet_list_open";
}
function isTodoItem(tokens, index) {
  if (index < 2) return false;
  const inlineToken = tokens[index];
  const paragraphToken = tokens[index - 1];
  const listItemToken = tokens[index - 2];
  if (!inlineToken || !paragraphToken || !listItemToken) return false;
  if (listItemToken.attrGet("data-bangle-list-kind") === "task") {
    return false;
  }
  return isInline(inlineToken) && isParagraphOpen(paragraphToken) && isListItemOpen(listItemToken) && startsWithTodoMarkdown(inlineToken.content);
}
function startsWithTodoMarkdown(content) {
  if (!content) return false;
  return content.startsWith("[ ] ") || content.startsWith("[x] ") || content.startsWith("[X] ");
}
function convertToTaskItem(tokens, index) {
  if (index < 2) return;
  const listItemOpen = tokens[index - 2];
  const inlineToken = tokens[index];
  if (!listItemOpen || !inlineToken || !inlineToken.children || inlineToken.children.length === 0)
    return;
  const firstTextChild = inlineToken.children.find(
    (child) => child.type === "text"
  );
  if (!firstTextChild || typeof firstTextChild.content !== "string") return;
  let markerFound = false;
  let isChecked = false;
  if (firstTextChild.content.startsWith("[ ] ")) {
    markerFound = true;
    isChecked = false;
  } else if (firstTextChild.content.startsWith("[x] ") || firstTextChild.content.startsWith("[X] ")) {
    markerFound = true;
    isChecked = true;
  }
  if (markerFound) {
    listItemOpen.attrSet("data-bangle-list-kind", "task");
    listItemOpen.attrSet(
      "data-bangle-task-checked",
      isChecked ? "true" : "false"
    );
    firstTextChild.content = firstTextChild.content.slice(4);
  }
}
function isInline(token) {
  return _optionalChain([token, 'optionalAccess', _4 => _4.type]) === "inline";
}
function isParagraphOpen(token) {
  return _optionalChain([token, 'optionalAccess', _5 => _5.type]) === "paragraph_open";
}
function isListItemOpen(token) {
  return _optionalChain([token, 'optionalAccess', _6 => _6.type]) === "list_item_open";
}



exports.listMarkdownPlugin = listMarkdownPlugin;
